@@ -2,7 +2,7 @@
  * main.c
  * main() routine, printer functions
  *
- * Copyright (c) 2011 pkgconf authors (see AUTHORS).
+ * Copyright (c) 2011, 2012, 2013 pkgconf authors (see AUTHORS).
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -44,11 +44,12 @@
 #define PKG_HELP			(1<<25)
 #define PKG_PRINT_ERRORS		(1<<26)
 #define PKG_SIMULATE			(1<<27)
+#define PKG_NO_CACHE			(1<<28)
 
 static unsigned int global_traverse_flags = PKGF_NONE;
 
 static int want_flags;
-static int maximum_traverse_depth = -1;
+static int maximum_traverse_depth = 2000;
 
 static char *want_variable = NULL;
 static char *sysroot_dir = NULL;
@@ -73,23 +74,6 @@ fragment_has_system_dir(pkg_fragment_t *frag)
 	return false;
 }
 
-static inline const char *
-print_sysroot_dir(pkg_fragment_t *frag)
-{
-	if (sysroot_dir == NULL)
-		return "";
-	else switch (frag->type)
-	{
-	case 'L':
-	case 'I':
-		return sysroot_dir;
-	default:
-		break;
-	}
-
-	return "";
-}
-
 static void
 print_fragment(pkg_fragment_t *frag)
 {
@@ -97,7 +81,7 @@ print_fragment(pkg_fragment_t *frag)
 		return;
 
 	if (frag->type)
-		printf("-%c%s%s ", frag->type, print_sysroot_dir(frag), frag->data);
+		printf("-%c%s ", frag->type, frag->data);
 	else
 		printf("%s ", frag->data);
 }
@@ -105,19 +89,21 @@ print_fragment(pkg_fragment_t *frag)
 static void
 print_list_entry(const pkg_t *entry)
 {
-	if (entry->uninstalled)
+	if (entry->flags & PKG_PROPF_UNINSTALLED)
 		return;
 
 	printf("%-30s %s - %s\n", entry->id, entry->realname, entry->description);
 }
 
 static void
-print_cflags(pkg_fragment_t *list)
+print_cflags(pkg_list_t *list)
 {
-	pkg_fragment_t *frag;
+	pkg_node_t *node;
 
-	PKG_FOREACH_LIST_ENTRY(list, frag)
+	PKG_FOREACH_LIST_ENTRY(list->head, node)
 	{
+		pkg_fragment_t *frag = node->data;
+
 		if ((want_flags & PKG_CFLAGS_ONLY_I) == PKG_CFLAGS_ONLY_I && frag->type != 'I')
 			continue;
 		else if ((want_flags & PKG_CFLAGS_ONLY_OTHER) == PKG_CFLAGS_ONLY_OTHER && frag->type == 'I')
@@ -128,12 +114,14 @@ print_cflags(pkg_fragment_t *list)
 }
 
 static void
-print_libs(pkg_fragment_t *list)
+print_libs(pkg_list_t *list)
 {
-	pkg_fragment_t *frag;
+	pkg_node_t *node;
 
-	PKG_FOREACH_LIST_ENTRY(list, frag)
+	PKG_FOREACH_LIST_ENTRY(list->head, node)
 	{
+		pkg_fragment_t *frag = node->data;
+
 		if ((want_flags & PKG_LIBS_ONLY_LDPATH) == PKG_LIBS_ONLY_LDPATH && frag->type != 'L')
 			continue;
 		else if ((want_flags & PKG_LIBS_ONLY_LIBNAME) == PKG_LIBS_ONLY_LIBNAME && frag->type != 'l')
@@ -158,25 +146,31 @@ print_modversion(pkg_t *pkg, void *unused, unsigned int flags)
 static void
 print_variables(pkg_t *pkg, void *unused, unsigned int flags)
 {
-	pkg_tuple_t *node;
+	pkg_node_t *node;
 	(void) unused;
 	(void) flags;
 
-	PKG_FOREACH_LIST_ENTRY(pkg->vars, node)
-		printf("%s\n", node->key);
+	PKG_FOREACH_LIST_ENTRY(pkg->vars.head, node)
+	{
+		pkg_tuple_t *tuple = node->data;
+
+		printf("%s\n", tuple->key);
+	}
 }
 
 static void
 print_requires(pkg_t *pkg)
 {
-	pkg_dependency_t *node;
+	pkg_node_t *node;
 
-	PKG_FOREACH_LIST_ENTRY(pkg->requires, node)
+	PKG_FOREACH_LIST_ENTRY(pkg->requires.head, node)
 	{
-		printf("%s", node->package);
+		pkg_dependency_t *dep = node->data;
 
-		if (node->version != NULL)
-			printf(" %s %s", pkg_get_comparator(node), node->version);
+		printf("%s", dep->package);
+
+		if (dep->version != NULL)
+			printf(" %s %s", pkg_get_comparator(dep), dep->version);
 
 		printf("\n");
 	}
@@ -185,14 +179,16 @@ print_requires(pkg_t *pkg)
 static void
 print_requires_private(pkg_t *pkg)
 {
-	pkg_dependency_t *node;
+	pkg_node_t *node;
 
-	PKG_FOREACH_LIST_ENTRY(pkg->requires_private, node)
+	PKG_FOREACH_LIST_ENTRY(pkg->requires_private.head, node)
 	{
-		printf("%s", node->package);
+		pkg_dependency_t *dep = node->data;
 
-		if (node->version != NULL)
-			printf(" %s %s", pkg_get_comparator(node), node->version);
+		printf("%s", dep->package);
+
+		if (dep->version != NULL)
+			printf(" %s %s", pkg_get_comparator(dep), dep->version);
 
 		printf("\n");
 	}
@@ -201,15 +197,17 @@ print_requires_private(pkg_t *pkg)
 static void
 print_digraph_node(pkg_t *pkg, void *unused, unsigned int flags)
 {
-	pkg_dependency_t *node;
+	pkg_node_t *node;
 	(void) unused;
 	(void) flags;
 
 	printf("\"%s\" [fontname=Sans fontsize=8]\n", pkg->id);
 
-	PKG_FOREACH_LIST_ENTRY(pkg->requires, node)
+	PKG_FOREACH_LIST_ENTRY(pkg->requires.head, node)
 	{
-		printf("\"%s\" -- \"%s\" [fontname=Sans fontsize=8]\n", node->package, pkg->id);
+		pkg_dependency_t *dep = node->data;
+
+		printf("\"%s\" -- \"%s\" [fontname=Sans fontsize=8]\n", dep->package, pkg->id);
 	}
 }
 
@@ -269,7 +267,7 @@ print_variable(pkg_t *pkg, void *data, unsigned int flags)
 	const char *var;
 	(void) flags;
 
-	var = pkg_tuple_find(pkg->vars, req->variable);
+	var = pkg_tuple_find(&pkg->vars, req->variable);
 	if (var != NULL)
 	{
 		if (*(req->buf) == '\0')
@@ -305,19 +303,19 @@ apply_variable(pkg_t *world, void *variable, int maxdepth, unsigned int flags)
 static bool
 apply_cflags(pkg_t *world, void *list_head, int maxdepth, unsigned int flags)
 {
-	pkg_fragment_t **head = list_head;
+	pkg_list_t *list = list_head;
 	int eflag;
 
-	eflag = pkg_cflags(world, head, maxdepth, flags | PKGF_SEARCH_PRIVATE);
+	eflag = pkg_cflags(world, list, maxdepth, flags | PKGF_SEARCH_PRIVATE);
 	if (eflag != PKG_ERRF_OK)
 		return false;
 
-	if (*head == NULL)
+	if (list->head == NULL)
 		return true;
 
-	print_cflags(*head);
+	print_cflags(list);
 
-	pkg_fragment_free(*head);
+	pkg_fragment_free(list);
 
 	return true;
 }
@@ -325,53 +323,57 @@ apply_cflags(pkg_t *world, void *list_head, int maxdepth, unsigned int flags)
 static bool
 apply_libs(pkg_t *world, void *list_head, int maxdepth, unsigned int flags)
 {
-	pkg_fragment_t **head = list_head;
+	pkg_list_t *list = list_head;
 	int eflag;
 
-	eflag = pkg_libs(world, head, maxdepth, flags);
+	eflag = pkg_libs(world, list, maxdepth, flags);
 	if (eflag != PKG_ERRF_OK)
 		return false;
 
-	if (*head == NULL)
+	if (list->head == NULL)
 		return true;
 
-	print_libs(*head);
+	print_libs(list);
+
+	pkg_fragment_free(list);
 
-	pkg_fragment_free(*head);
 	return true;
 }
 
 static bool
 apply_requires(pkg_t *world, void *unused, int maxdepth, unsigned int flags)
 {
-	pkg_dependency_t *iter;
+	pkg_node_t *iter;
 	(void) unused;
 	(void) maxdepth;
 
-	PKG_FOREACH_LIST_ENTRY(world->requires, iter)
+	PKG_FOREACH_LIST_ENTRY(world->requires.head, iter)
 	{
 		pkg_t *pkg;
+		pkg_dependency_t *dep = iter->data;
 
-		pkg = pkg_verify_dependency(iter, flags, NULL);
+		pkg = pkg_verify_dependency(dep, flags, NULL);
 		print_requires(pkg);
 
 		pkg_free(pkg);
 	}
+
 	return true;
 }
 
 static bool
 apply_requires_private(pkg_t *world, void *unused, int maxdepth, unsigned int flags)
 {
-	pkg_dependency_t *iter;
+	pkg_node_t *iter;
 	(void) unused;
 	(void) maxdepth;
 
-	PKG_FOREACH_LIST_ENTRY(world->requires, iter)
+	PKG_FOREACH_LIST_ENTRY(world->requires.head, iter)
 	{
 		pkg_t *pkg;
+		pkg_dependency_t *dep = iter->data;
 
-		pkg = pkg_verify_dependency(iter, flags | PKGF_SEARCH_PRIVATE, NULL);
+		pkg = pkg_verify_dependency(dep, flags | PKGF_SEARCH_PRIVATE, NULL);
 		print_requires_private(pkg);
 
 		pkg_free(pkg);
@@ -385,7 +387,7 @@ check_uninstalled(pkg_t *pkg, void *data, unsigned int flags)
 	int *retval = data;
 	(void) flags;
 
-	if (pkg->uninstalled)
+	if (pkg->flags & PKG_PROPF_UNINSTALLED)
 		*retval = EXIT_SUCCESS;
 }
 
@@ -405,10 +407,32 @@ apply_uninstalled(pkg_t *world, void *data, int maxdepth, unsigned int flags)
 static void
 print_graph_node(pkg_t *pkg, void *data, unsigned int flags)
 {
+	pkg_node_t *n;
+
 	(void) data;
 	(void) flags;
 
-	printf("Considering graph node '%s' (%p)\n", pkg->id, pkg);
+	printf("node '%s' {\n", pkg->id);
+
+	if (pkg->version != NULL)
+		printf("    version = '%s';\n", pkg->version);
+
+	PKG_FOREACH_LIST_ENTRY(pkg->requires.head, n)
+	{
+		pkg_dependency_t *dep = n->data;
+		printf("    dependency '%s'", dep->package);
+		if (dep->compare != PKG_ANY)
+		{
+			printf(" {\n");
+			printf("        comparator = '%s';\n", pkg_get_comparator(dep));
+			printf("        version = '%s';\n", dep->version);
+			printf("    };\n");
+		}
+		else
+			printf(";\n");
+	}
+
+	printf("};\n");
 }
 
 static bool
@@ -434,7 +458,7 @@ static void
 about(void)
 {
 	printf("%s %s%s\n", PACKAGE_NAME, PACKAGE_VERSION, HAVE_STRICT_MODE ? " [strict]" : " [pkg-config " PKG_PKGCONFIG_VERSION_EQUIV " compatible]");
-	printf("Copyright (c) 2011 - 2012 pkgconf authors (see AUTHORS in documentation directory).\n\n");
+	printf("Copyright (c) 2011, 2012, 2013 pkgconf authors (see AUTHORS in documentation directory).\n\n");
 	printf("Permission to use, copy, modify, and/or distribute this software for any\n");
 	printf("purpose with or without fee is hereby granted, provided that the above\n");
 	printf("copyright notice and this permission notice appear in all copies.\n\n");
@@ -460,6 +484,8 @@ usage(void)
 	printf("  --silence-errors                  explicitly be silent about errors\n");
 	printf("  --list-all                        list all known packages\n");
 	printf("  --simulate                        simulate walking the calculated dependency graph\n");
+	printf("  --no-cache                        do not cache already seen packages when\n");
+	printf("                                    walking the dependency graph\n");
 
 	printf("\nchecking specific pkg-config database entries:\n\n");
 
@@ -501,8 +527,7 @@ int
 main(int argc, char *argv[])
 {
 	int ret;
-	pkg_queue_t *pkgq = NULL;
-	pkg_queue_t *pkgq_head = NULL;
+	pkg_list_t pkgq = PKG_LIST_INITIALIZER;
 	char *builddir;
 	char *required_pkgconfig_version = NULL;
 	char *required_exact_module_version = NULL;
@@ -548,6 +573,7 @@ main(int argc, char *argv[])
 		{ "silence-errors", no_argument, &want_flags, PKG_SILENCE_ERRORS, },
 		{ "list-all", no_argument, &want_flags, PKG_LIST|PKG_PRINT_ERRORS, },
 		{ "simulate", no_argument, &want_flags, PKG_SIMULATE, },
+		{ "no-cache", no_argument, &want_flags, PKG_NO_CACHE, },
 		{ NULL, 0, NULL, 0 }
 	};
 
@@ -634,6 +660,9 @@ main(int argc, char *argv[])
 	if ((want_flags & PKG_ENV_ONLY) == PKG_ENV_ONLY)
 		global_traverse_flags |= PKGF_ENV_ONLY;
 
+	if ((want_flags & PKG_NO_CACHE) == PKG_NO_CACHE)
+		global_traverse_flags |= PKGF_NO_CACHE;
+
 	if ((want_flags & PKG_NO_UNINSTALLED) == PKG_NO_UNINSTALLED || getenv("PKG_CONFIG_DISABLE_UNINSTALLED") != NULL)
 		global_traverse_flags |= PKGF_NO_UNINSTALLED;
 
@@ -649,7 +678,10 @@ main(int argc, char *argv[])
 		pkg_tuple_add_global("pc_top_builddir", "$(top_builddir)");
 
 	if ((sysroot_dir = getenv("PKG_CONFIG_SYSROOT_DIR")) != NULL)
+	{
 		pkg_tuple_add_global("pc_sysrootdir", sysroot_dir);
+		global_traverse_flags |= PKGF_MUNGE_SYSROOT_PREFIX;
+	}
 	else
 		pkg_tuple_add_global("pc_sysrootdir", "/");
 
@@ -670,16 +702,19 @@ main(int argc, char *argv[])
 	if (required_module_version != NULL)
 	{
 		pkg_t *pkg;
-		pkg_dependency_t *pkghead = NULL, *pkgiter = NULL;
+		pkg_node_t *node;
+		pkg_list_t deplist = PKG_LIST_INITIALIZER;
 
 		while (argv[pkg_optind])
 		{
-			pkghead = pkg_dependency_parse_str(pkghead, argv[pkg_optind]);
+			pkg_dependency_parse_str(&deplist, argv[pkg_optind]);
 			pkg_optind++;
 		}
 
-		PKG_FOREACH_LIST_ENTRY(pkghead, pkgiter)
+		PKG_FOREACH_LIST_ENTRY(deplist.head, node)
 		{
+			pkg_dependency_t *pkgiter = node->data;
+
 			pkg = pkg_find(pkgiter->package, global_traverse_flags);
 			if (pkg == NULL)
 				return EXIT_FAILURE;
@@ -694,16 +729,19 @@ main(int argc, char *argv[])
 	if (required_exact_module_version != NULL)
 	{
 		pkg_t *pkg;
-		pkg_dependency_t *pkghead = NULL, *pkgiter = NULL;
+		pkg_node_t *node;
+		pkg_list_t deplist = PKG_LIST_INITIALIZER;
 
 		while (argv[pkg_optind])
 		{
-			pkghead = pkg_dependency_parse_str(pkghead, argv[pkg_optind]);
+			pkg_dependency_parse_str(&deplist, argv[pkg_optind]);
 			pkg_optind++;
 		}
 
-		PKG_FOREACH_LIST_ENTRY(pkghead, pkgiter)
+		PKG_FOREACH_LIST_ENTRY(deplist.head, node)
 		{
+			pkg_dependency_t *pkgiter = node->data;
+
 			pkg = pkg_find(pkgiter->package, global_traverse_flags);
 			if (pkg == NULL)
 				return EXIT_FAILURE;
@@ -718,16 +756,19 @@ main(int argc, char *argv[])
 	if (required_max_module_version != NULL)
 	{
 		pkg_t *pkg;
-		pkg_dependency_t *pkghead = NULL, *pkgiter = NULL;
+		pkg_node_t *node;
+		pkg_list_t deplist = PKG_LIST_INITIALIZER;
 
 		while (argv[pkg_optind])
 		{
-			pkghead = pkg_dependency_parse_str(pkghead, argv[pkg_optind]);
+			pkg_dependency_parse_str(&deplist, argv[pkg_optind]);
 			pkg_optind++;
 		}
 
-		PKG_FOREACH_LIST_ENTRY(pkghead, pkgiter)
+		PKG_FOREACH_LIST_ENTRY(deplist.head, node)
 		{
+			pkg_dependency_t *pkgiter = node->data;
+
 			pkg = pkg_find(pkgiter->package, global_traverse_flags);
 			if (pkg == NULL)
 				return EXIT_FAILURE;
@@ -757,11 +798,7 @@ main(int argc, char *argv[])
 
 		if (argv[pkg_optind + 1] == NULL || !PKG_OPERATOR_CHAR(*(argv[pkg_optind + 1])))
 		{
-			pkgq = pkg_queue_push(pkgq, package);
-
-			if (pkgq_head == NULL)
-				pkgq_head = pkgq;
-
+			pkg_queue_push(&pkgq, package);
 			pkg_optind++;
 		}
 		else
@@ -771,14 +808,11 @@ main(int argc, char *argv[])
 			snprintf(packagebuf, sizeof packagebuf, "%s %s %s", package, argv[pkg_optind + 1], argv[pkg_optind + 2]);
 			pkg_optind += 3;
 
-			pkgq = pkg_queue_push(pkgq, packagebuf);
-
-			if (pkgq_head == NULL)
-				pkgq_head = pkgq;
+			pkg_queue_push(&pkgq, packagebuf);
 		}
 	}
 
-	if (pkgq_head == NULL)
+	if (pkgq.head == NULL)
 	{
 		fprintf(stderr, "Please specify at least one package name on the command line.\n");
 		return EXIT_FAILURE;
@@ -786,7 +820,18 @@ main(int argc, char *argv[])
 
 	ret = EXIT_SUCCESS;
 
-	if (!pkg_queue_validate(pkgq_head, maximum_traverse_depth, global_traverse_flags))
+	if ((want_flags & PKG_SIMULATE) == PKG_SIMULATE)
+	{
+		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
+
+		if (!pkg_queue_apply(&pkgq, apply_simulate, -1, global_traverse_flags | PKGF_SKIP_ERRORS, NULL))
+		{
+			ret = EXIT_FAILURE;
+			goto out;
+		}
+	}
+
+	if (!pkg_queue_validate(&pkgq, maximum_traverse_depth, global_traverse_flags))
 	{
 		ret = EXIT_FAILURE;
 		goto out;
@@ -795,7 +840,7 @@ main(int argc, char *argv[])
 	if ((want_flags & PKG_UNINSTALLED) == PKG_UNINSTALLED)
 	{
 		ret = EXIT_FAILURE;
-		pkg_queue_apply(pkgq_head, apply_uninstalled, maximum_traverse_depth, global_traverse_flags, &ret);
+		pkg_queue_apply(&pkgq, apply_uninstalled, maximum_traverse_depth, global_traverse_flags, &ret);
 		goto out;
 	}
 
@@ -803,18 +848,7 @@ main(int argc, char *argv[])
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
-		if (!pkg_queue_apply(pkgq_head, apply_digraph, maximum_traverse_depth, global_traverse_flags, NULL))
-		{
-			ret = EXIT_FAILURE;
-			goto out;
-		}
-	}
-
-	if ((want_flags & PKG_SIMULATE) == PKG_SIMULATE)
-	{
-		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-
-		if (!pkg_queue_apply(pkgq_head, apply_simulate, -1, global_traverse_flags, NULL))
+		if (!pkg_queue_apply(&pkgq, apply_digraph, maximum_traverse_depth, global_traverse_flags, NULL))
 		{
 			ret = EXIT_FAILURE;
 			goto out;
@@ -825,7 +859,7 @@ main(int argc, char *argv[])
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
-		if (!pkg_queue_apply(pkgq_head, apply_modversion, 2, global_traverse_flags, NULL))
+		if (!pkg_queue_apply(&pkgq, apply_modversion, 2, global_traverse_flags, NULL))
 		{
 			ret = EXIT_FAILURE;
 			goto out;
@@ -836,7 +870,7 @@ main(int argc, char *argv[])
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
-		if (!pkg_queue_apply(pkgq_head, apply_variables, 2, global_traverse_flags, NULL))
+		if (!pkg_queue_apply(&pkgq, apply_variables, 2, global_traverse_flags, NULL))
 		{
 			ret = EXIT_FAILURE;
 			goto out;
@@ -847,7 +881,7 @@ main(int argc, char *argv[])
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
-		if (!pkg_queue_apply(pkgq_head, apply_variable, 2, global_traverse_flags | PKGF_SKIP_ROOT_VIRTUAL, want_variable))
+		if (!pkg_queue_apply(&pkgq, apply_variable, 2, global_traverse_flags | PKGF_SKIP_ROOT_VIRTUAL, want_variable))
 		{
 			ret = EXIT_FAILURE;
 			goto out;
@@ -858,7 +892,7 @@ main(int argc, char *argv[])
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
-		if (!pkg_queue_apply(pkgq_head, apply_requires, maximum_traverse_depth, global_traverse_flags, NULL))
+		if (!pkg_queue_apply(&pkgq, apply_requires, maximum_traverse_depth, global_traverse_flags, NULL))
 		{
 			ret = EXIT_FAILURE;
 			goto out;
@@ -869,7 +903,7 @@ main(int argc, char *argv[])
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
-		if (!pkg_queue_apply(pkgq_head, apply_requires_private, maximum_traverse_depth, global_traverse_flags, NULL))
+		if (!pkg_queue_apply(&pkgq, apply_requires_private, maximum_traverse_depth, global_traverse_flags, NULL))
 		{
 			ret = EXIT_FAILURE;
 			goto out;
@@ -878,9 +912,9 @@ main(int argc, char *argv[])
 
 	if ((want_flags & PKG_CFLAGS) == PKG_CFLAGS)
 	{
-		pkg_fragment_t *frag_list = NULL;
+		pkg_list_t frag_list = PKG_LIST_INITIALIZER;
 
-		if (!pkg_queue_apply(pkgq_head, apply_cflags, maximum_traverse_depth, global_traverse_flags, &frag_list))
+		if (!pkg_queue_apply(&pkgq, apply_cflags, maximum_traverse_depth, global_traverse_flags, &frag_list))
 		{
 			ret = EXIT_FAILURE;
 			goto out_println;
@@ -889,16 +923,16 @@ main(int argc, char *argv[])
 
 	if ((want_flags & PKG_LIBS) == PKG_LIBS)
 	{
-		pkg_fragment_t *frag_list = NULL;
+		pkg_list_t frag_list = PKG_LIST_INITIALIZER;
 
-		if (!pkg_queue_apply(pkgq_head, apply_libs, maximum_traverse_depth, global_traverse_flags, &frag_list))
+		if (!pkg_queue_apply(&pkgq, apply_libs, maximum_traverse_depth, global_traverse_flags, &frag_list))
 		{
 			ret = EXIT_FAILURE;
 			goto out_println;
 		}
 	}
 
-	pkg_queue_free(pkgq_head);
+	pkg_queue_free(&pkgq);
 
 out_println:
 	if (want_flags & (PKG_CFLAGS|PKG_LIBS))
@@ -906,5 +940,7 @@ out_println:
 
 out:
 	pkg_tuple_free_global();
+	pkg_cache_free();
+
 	return ret;
 }
